@@ -5,10 +5,10 @@ import itertools
 from shutil import copyfile
 
 
-from imagepreprocessing.internal_functions.file_operations import __read_from_file, __write_to_file
-from imagepreprocessing.internal_functions.convert_annotations import __convert_annotations_opencv_to_yolo, __convert_annotations_yolo_to_opencv
-from imagepreprocessing.internal_functions.other_functions import __run_shell_command
-from imagepreprocessing.internal_functions.cfg_templates import __get_cfg_template
+from imagepreprocessing.__file_operations import __read_from_file, __write_to_file
+from imagepreprocessing.__convert_annotations import __convert_annotations_opencv_to_yolo, __convert_annotations_yolo_to_opencv
+from imagepreprocessing.__other_functions import __run_shell_command
+from imagepreprocessing.__cfg_templates import __get_cfg_template
 
 
 # yolo functions
@@ -268,11 +268,20 @@ def yolo_annotation_tool(images_path, class_names_file, max_windows_size=(1200,7
     images = os.listdir(images_path)
     images.sort()
 
+
     # remove not included files
-    for image in images:
-        image_name, image_extension = os.path.splitext(image)
-        if image_extension not in image_extensions: 
-            images.remove(image)        
+    # for image in images:
+    #     image_name, image_extension = os.path.splitext(image)
+    #     if image_extension not in image_extensions: 
+    #         images.remove(image)        
+
+    def __filter_function(image):
+        _, image_extension = os.path.splitext(image)
+        if image_extension in image_extensions: 
+            return image
+
+    images = filter(__filter_function, images)        
+
 
     # add paths to images
     images = [os.path.join(images_path, image) for image in images]
@@ -453,7 +462,7 @@ def yolo_annotation_tool(images_path, class_names_file, max_windows_size=(1200,7
     global __drawing__
     __drawing__ = False
     window_name = "Yolo annotation tool"
-
+    image = images[0]
 
     # create window and set it up
     cv2.namedWindow(window_name)
@@ -659,10 +668,97 @@ def draw_bounding_boxes(images_path_file, class_names_file, save_path = "annoted
         print("Image saved: {0}".format(new_file_save_path))
 
 
+def count_classes_from_annotation_files(class_path, names_path, include_zeros = False):
+    """
+    Counts individual class appearances in a folder.
+
+    # Arguments:
+        class_path: path of a class folder. (a folder with annotation .txt files)
+        names_path: path of the obj.names file.
+        include_zeros: (False) includes non existing classes to the dictionary.
+    """
+
+    classes = {}
 
 
+    # read files
+    names = __read_from_file(names_path).split()
+    annotation_files = os.listdir(class_path)
+    annotation_files = list(filter(lambda x: x.endswith(".txt"), annotation_files))
+    
+    # include zeros
+    if(include_zeros):
+        for name in names:
+            classes.update({name : 0})          
+
+    for annotation_file in annotation_files:
+        annotations_str = __read_from_file(os.path.join(class_path, annotation_file))
+
+        # split annotations
+        annotations = annotations_str.split("\n")
+        annotations = filter(None, annotations)
+        annotations = [annotation.split(" ") for annotation in annotations]
+
+        # count classes
+        for annotation in annotations:
+            current_class_index = int(annotation[0])
+            current_class_name = names[current_class_index]
+            if(current_class_name in classes):
+                classes.update({current_class_name : classes[current_class_name] + 1})
+            else:
+                classes.update({current_class_name : 1})
+
+    return classes
 
 
+def remove_class_from_annotation_files(class_path, class_index_to_remove, new_annotations_path = "new_annotations"):
+    """
+    Removes a class and fixes the indexes of all annotation files in a directory
+
+    # Arguments:
+        class_path: path of a class folder. (a folder with annotation .txt files)
+        class_index_to_remove: index of the class which will be removed.
+        new_annotations_path ("new_annotations"): save path for new annotation files. (you can give the same path to override or another path for keeping the original ones)
+    """
+
+    # read and filter annotation files
+    annotation_files = os.listdir(class_path)
+    annotation_files = list(filter(lambda x: x.endswith(".txt"), annotation_files))
+
+    # create new annotations dir if not exists
+    if(not os.path.exists(new_annotations_path)):
+        os.makedirs(new_annotations_path)
+
+    # loop annotation files
+    for annotation_file in annotation_files:
+        annotations_str = __read_from_file(os.path.join(class_path, annotation_file))
+
+        # split annotations
+        annotations = annotations_str.split("\n")
+        annotations = filter(None, annotations)
+        annotations = [annotation.split(" ") for annotation in annotations]
+
+        # create new annotations by decreasing bigger indexes by one
+        new_annotations = []
+        for annotation in annotations:
+            current_class_index = int(annotation[0])
+            if(current_class_index < class_index_to_remove):
+                new_annotations.append(annotation)
+            elif(current_class_index > class_index_to_remove):
+                # temp_annotation = annotation
+                annotation[0] = int(annotation[0]) - 1
+                new_annotations.append(annotation)
+
+        # prepare annotations to write a file
+        write_ready_annotations = []
+        for new_annotation in new_annotations:
+            write_ready_annotations.append("{0} {1} {2} {3} {4}".format(new_annotation[0], new_annotation[1], new_annotation[2], new_annotation[3], new_annotation[4]))
+
+        # new path
+        annotation_path = os.path.join(new_annotations_path, annotation_file)
+
+        # write
+        __write_to_file(write_ready_annotations, annotation_path)
 
 
 
